# generate.uriskip.sh
# -------------------
#
# This script generate an uribl skip exclusion for whitelisted domains
# useful to reduce dns queries
# MailScanner can be restarted after run
# put this pile into /batch folder and schedule into cron periodically
# 01 21 * * 2 /batch/generate.uriskip.sh


# Parameters
MAILFOLDER=/var/spool/MailScanner/quarantine
MINOCCURRENCES=3
TEMPFILEPATTERN=/batch/generate.uriskip
LOGFILE=/batch/generate.uriskip.log
EXCLUSIONFILE=/etc/mail/spamassassin/comet.uriskip.cf
DOMPERLINES=10
WL="dwl.dnswl.org;white.uribl.com;iddb.isipp.com;white.dnsbl.brukalai.lt;dnswl.spfbl.net"



start_time="$(date -u +%s)"

# Extracting urls from emails
grep -REoh '(http|https):\/\/[^\/|\=i|\>|\<|\@"|\?|\(|\)]+' $MAILFOLDER > $TEMPFILEPATTERN.1


# Sorting unique limiting occurrences
cat $TEMPFILEPATTERN.1 | sort | uniq -cd | awk -v limit=$MINOCCURRENCES '$1 > limit{print $2}' > $TEMPFILEPATTERN.2


# Removing protocol prefix and exctracting second level domains
cat $TEMPFILEPATTERN.2 | sed -e 's/http:\/\///g' -e 's/https:\/\///g' -e 's/ftp:\/\///g' | grep -Eo '([^.\s]+\.[^.\s]+)$' |sort | uniq > $TEMPFILEPATTERN.3


# Nslookup take only valid domainsa
echo "# Uribl eclusion file generated at : $(date)" > $LOGFILE
echo "" >> $LOGFILE
rm -f $TEMPFILEPATTERN.4
while read domline
do
 y=$(echo $domline | grep -P "^([a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,}$")
 if ((${#y} != 0 )) ; then
  r=`nslookup $domline | grep "NXDOMAIN"`
  if ((${#r} == 0)) ; then
   echo "Regex & nslookup valid domain : " $domline >> $LOGFILE
   echo $domline >> $TEMPFILEPATTERN.4
  fi
 fi
done < $TEMPFILEPATTERN.3


# Search each domain into whitelists given and write uriskip exclusion file
ix=0;ir=0;is="";rm -f $TEMPFILEPATTERN.5
echo "# Uribl eclusion file generated at : $(date)" > $EXCLUSIONFILE
echo "" >> $EXCLUSIONFILE
echo "" >> $LOGFILE
while read domline
do
 rr=0;
 echo "Searching whitelist for domain : " $domline >> $LOGFILE
 for i in $(echo $WL | sed "s/;/ /g")
 do
  r=$(dig +noall +answer +short $domline.$i)
  if ((${#r} > 0)) ; then
   rr=1
   echo " - Found in whitelist $domline.$i" >> $LOGFILE
  fi
 done
 if [ $rr = 1 ]; then
  let "ix=ix+1";let "ir=ir+1"
  echo " - ADD $domline in uriskip file" >> $LOGFILE
  is="$is $domline"
  if [ $ir = $DOMPERLINES ]; then
   echo "uridnsbl_skip_domain $is" >> $EXCLUSIONFILE
   ir=0;is=""
  fi
 fi
done < $TEMPFILEPATTERN.4
if [ $ir != 0 ]; then
 echo "uridnsbl_skip_domain $is" >> $EXCLUSIONFILE
 ir=0;is=""
fi


# Write results into log
echo "" >> $LOGFILE
echo "$ix domains inserted into exclusion cf file" >> $LOGFILE


# Restart mailscanner
# service mailscanner restart


# Remove tempfiles
rm -f $TEMPFILEPATTERN.1
rm -f $TEMPFILEPATTERN.2
rm -f $TEMPFILEPATTERN.3
rm -f $TEMPFILEPATTERN.4


# Timers in log
end_time="$(date -u +%s)"
elapsed="$(($end_time-$start_time))"
echo "Total of $elapsed seconds elapsed for process" >> $LOGFILE


